use async_graphql::{Context, Object};
use como_domain::{
    item::{queries::GetItemQuery, ItemDto, ItemState},
    projects::queries::GetProjectQuery,
};
use como_infrastructure::register::ServiceRegister;
use uuid::Uuid;

use crate::projects::Project;

pub struct CreatedItem {
    pub id: Uuid,
}

#[Object]
impl CreatedItem {
    pub async fn item(&self, ctx: &Context<'_>) -> anyhow::Result<Item> {
        let item = ctx
            .data_unchecked::<ServiceRegister>()
            .item_service
            .get_item(GetItemQuery { item_id: self.id })
            .await?;

        Ok(item.into())
    }
}

pub struct Item {
    pub id: Uuid,
    pub title: String,
    pub description: Option<String>,
    pub state: ItemState,
}

#[Object]
impl Item {
    pub async fn id(&self, _ctx: &Context<'_>) -> anyhow::Result<Uuid> {
        Ok(self.id)
    }

    pub async fn title(&self, _ctx: &Context<'_>) -> anyhow::Result<String> {
        Ok(self.title.clone())
    }

    pub async fn description(&self, _ctx: &Context<'_>) -> anyhow::Result<Option<String>> {
        Ok(self.description.clone())
    }

    pub async fn state(&self, _ctx: &Context<'_>) -> anyhow::Result<ItemState> {
        Ok(self.state)
    }

    pub async fn project(&self, ctx: &Context<'_>) -> anyhow::Result<Project> {
        let project = ctx
            .data_unchecked::<ServiceRegister>()
            .project_service
            .get_project(GetProjectQuery {
                item_id: Some(self.id),
                project_id: None,
            })
            .await?;

        Ok(project.into())
    }
}

impl From<ItemDto> for Item {
    fn from(dto: ItemDto) -> Self {
        Self {
            id: dto.id,
            title: dto.title,
            description: dto.description,
            state: dto.state,
        }
    }
}
