use std::{collections::HashMap, sync::Arc};

use anyhow::Context;
use axum::async_trait;
use como_core::projects::ProjectService;
use como_domain::projects::{
    queries::{GetProjectQuery, GetProjectsQuery},
    ProjectDto,
};
use tokio::sync::Mutex;

pub struct DefaultProjectService {}

impl DefaultProjectService {
    pub fn new() -> Self {
        Self {}
    }
}

impl Default for DefaultProjectService {
    fn default() -> Self {
        Self::new()
    }
}

#[async_trait]
impl ProjectService for DefaultProjectService {
    async fn get_project(&self, _query: GetProjectQuery) -> anyhow::Result<ProjectDto> {
        todo!()
    }
    async fn get_projects(&self, _query: GetProjectsQuery) -> anyhow::Result<Vec<ProjectDto>> {
        todo!()
    }
}

pub struct MemoryProjectService {
    project_store: Arc<Mutex<HashMap<String, ProjectDto>>>,
}

impl MemoryProjectService {
    pub fn new() -> Self {
        Self {
            project_store: Arc::new(Mutex::new(HashMap::new())),
        }
    }
}

impl Default for MemoryProjectService {
    fn default() -> Self {
        Self::new()
    }
}

#[async_trait]
impl ProjectService for MemoryProjectService {
    async fn get_project(&self, query: GetProjectQuery) -> anyhow::Result<ProjectDto> {
        let ps = self.project_store.lock().await;
        if let Some(item_id) = query.item_id {
            Ok(ps
                .get(&item_id.to_string())
                .context("could not find project")?
                .clone())
        } else {
            Err(anyhow::anyhow!("could not find project"))
        }
    }
    async fn get_projects(&self, _query: GetProjectsQuery) -> anyhow::Result<Vec<ProjectDto>> {
        todo!()
    }
}
